module fifo_ctrl(clk, chan, val, trig, tx);
   input clk;

   input [3:0] chan;
   input [7:0] val;
   input       trig;

   // UART electrical interface
   output 	tx;
   
   // --------------------------------------------
   // registered input
   // --------------------------------------------
   reg [9:0]   val_r;
   reg [3:0]   chan_r;
   wire [7:0]  n2_hex = val_r[9:8] + ((val_r[9:8] < 4'hA) ? 8'd48 : 8'd87);
   wire [7:0]  n1_hex = val_r[7:4] + ((val_r[7:4] < 4'hA) ? 8'd48 : 8'd87);
   wire [7:0]  n0_hex = val_r[3:0] + ((val_r[3:0] < 4'hA) ? 8'd48 : 8'd87);
   
   // --------------------------------------------
   // fifo
   // --------------------------------------------
   reg [7:0]   fifo_in;   
   wire [7:0] fifoOut_uartIn;
   reg 	      fifo_wr = 0;
   reg 	      fifo_rd = 0;
   wire       fifo_empty;
	      
   fifo myFifo(.clk(clk), .rst(1'b0), 
	       .buf_in(fifo_in), .buf_out(fifoOut_uartIn), 
	       .wr_en(fifo_wr), .rd_en(fifo_rd), 
	       .buf_empty(fifo_empty), .buf_full(), .fifo_counter());
   
   // --------------------------------------------
   // uart
   // --------------------------------------------
   reg 	      uart_trigTx;
   wire       uart_busy;   
   wire       uart_trigRx;   
   wire [7:0] uart_rxByte;
   
   uart myUart(.clk(clk), .rst(1'b0),
	       .rx(), .tx(tx),
	       .transmit(uart_trigTx), .tx_byte(fifoOut_uartIn), 
	       .received(uart_trigRx), .rx_byte(uart_rxByte),	       
	       .is_transmitting(uart_busy), .is_receiving(),
	       .recv_error());
   
   // --------------------------------------------
   // self
   // --------------------------------------------   
   reg [3:0]   sIn = 4'd0;
   reg 	       sOut = 1'b0;   	       
   (* FSM_ENCODING="ONEHOT" *)
   always @(posedge clk) begin
      if (uart_trigTx) begin
	 $display("uart %d, '%c'", fifoOut_uartIn, fifoOut_uartIn);	       
      end
      
      // ***************************************************
      // write
      // ***************************************************
      fifo_wr <= 1'b0;      
      (* PARALLEL_CASE, FULL_CASE *) 
	case (sIn)       
	  4'd0: begin
	     // --- wait for trig; register inputs ---
	     if (trig) begin
		chan_r <= chan;
	      val_r <= val;
	      sIn <= sIn + 4'd1;
	   end // if triggered
	end
	4'd1: begin
	   fifo_in <= n1_hex;
	   fifo_wr <= 1'b1;
	   sIn <= sIn + 4'd1;
	end
	4'd2: begin
	   fifo_in <= n0_hex;
	   fifo_wr <= 1'b1;
	   sIn <= sIn + 4'd1;
	end
	4'd3: begin
	   if (chan_r == 'hF) begin
	      // carriage return...
	      fifo_in <= 8'd13;
	      sIn <= sIn + 4'd1;
	   end else begin 
	      fifo_in <= 8'd32;	
	      sIn <= 0;   
	   end	      
	   fifo_wr <= 1'b1;
	end
	4'd4: begin
	   // ... line feed
	   fifo_in <= 8'd10;
	   fifo_wr <= 1'b1;
	   sIn <= 0;
	end
      endcase
      
      // ***************************************************
      // read FIFO => UART Tx
      // ***************************************************
      uart_trigTx <= 1'b0;	     
      fifo_rd <= 1'b0;
      
      case (sOut)
	1'd0:
	  // wait for next byte
	  if (fifo_empty == 1'b0)
	    begin
	       // pop from FIFO
	       sOut <= 1'd1;		  
	       fifo_rd <= 1'b1;		  
	    end
	1'd1:
	  // start UART
	  if (uart_busy == 1'b0)
	    begin
	       sOut <= 1'd0;
	       uart_trigTx <= 1'b1;	       
	    end
      endcase
   end   

   always @(posedge clk) begin
      if (fifo_wr) begin
	 //$display("fifo ", fifo_in);	 
      end   
   end   
endmodule
